package com.chinamobile.xxjs.dios.bcm.client;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.chinamobile.xxjs.dios.bcm.client.api.ApiConfiguration;
import com.chinamobile.xxjs.dios.bcm.client.component.ClientTokenHandler;
import com.chinamobile.xxjs.dios.bcm.client.util.AppContext;
import java.io.IOException;
import java.security.SecureRandom;
import java.util.Objects;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.http.HttpServletRequest;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

@Configuration
@Import({ApiConfiguration.class, AppContext.class, ClientTokenHandler.class})
public class ClientConfiguration {
  @Bean
  HttpLoggingInterceptor httpLoggingInterceptor() {
    HttpLoggingInterceptor loggingInterceptor = new HttpLoggingInterceptor();
    loggingInterceptor.level(HttpLoggingInterceptor.Level.BODY);
    return loggingInterceptor;
  }
  
  @Bean
  Interceptor clientSecurityInterceptor(ClientTokenHandler clientTokenHandler) {
    return chain -> {
        Request request = chain.request();
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        if (Objects.isNull(requestAttributes)) {
          String clientToken = clientTokenHandler.getClientToken(request);
          if (clientToken != null)
            request = request.newBuilder().header("Authorization", clientToken).build(); 
        } else if (requestAttributes instanceof ServletRequestAttributes) {
          HttpServletRequest servletRequest = ((ServletRequestAttributes)requestAttributes).getRequest();
          String authorization = servletRequest.getHeader("Authorization");
          if (authorization != null) {
            request = request.newBuilder().header("Authorization", authorization).build();
          } else {
            String clientToken = clientTokenHandler.getClientToken(request);
            if (clientToken != null)
              request = request.newBuilder().header("Authorization", clientToken).build(); 
          } 
        } else if (requestAttributes == null) {
          String clientToken = clientTokenHandler.getClientToken(request);
          if (clientToken != null)
            request = request.newBuilder().header("Authorization", clientToken).build(); 
        } 
        return chain.proceed(request);
      };
  }
  
  @Bean
  OkHttpClient okHttpClient(ClientTokenHandler clientTokenHandler) throws Exception {
    TrustManager[] trustAllCerts = { (TrustManager)new Object(this) };
    SSLContext sslContext = SSLContext.getInstance("SSL");
    sslContext.init(null, trustAllCerts, new SecureRandom());
    OkHttpClient.Builder builder = new OkHttpClient.Builder();
    builder.addInterceptor(clientSecurityInterceptor(clientTokenHandler));
    builder.addInterceptor((Interceptor)httpLoggingInterceptor());
    builder.sslSocketFactory(sslContext.getSocketFactory(), (X509TrustManager)trustAllCerts[0]);
    HttpsURLConnection.setDefaultSSLSocketFactory(sslContext.getSocketFactory());
    HttpsURLConnection.setDefaultHostnameVerifier((s, sslSession) -> true);
    return builder.build();
  }
}
